'use strict';

define([
	'libraries/utils',
	'libraries/permissions',
	'libraries/consts'
], function (UTILS, PERMISSIONS, CONSTS) {

	var app = angular.module("JetBackupApp", [
		"ngRoute",
		"ngAnimate",
		"ngSanitize",
		"ngMaterial",
		"ui.bootstrap",
		"permission",
		"permission.ng",
		"angular-loading-bar",
		"oitozero.ngSweetAlert",
		"moment-picker",
		"chart.js",
		"jm.i18next"
	]).run(["$http", "PermPermissionStore", function($http, PermPermissionStore) {

		for(var i in PERMISSIONS) {
			if(!/^can/.test(i) && !/^is/.test(i) && !/^has/.test(i)) continue;
			PermPermissionStore.definePermission(i, function(name) { return PERMISSIONS[name]; });
		}
	}]).config([
		'$routeProvider', '$httpProvider', '$animateProvider', '$controllerProvider', '$compileProvider', '$filterProvider', '$permissionProvider', '$provide', 'cfpLoadingBarProvider', 'ChartJsProvider',
		function ($routeProvider, $httpProvider, $animateProvider, $controllerProvider, $compileProvider, $filterProvider, $permissionProvider, $provide, cfpLoadingBarProvider, ChartJsProvider) {
			$animateProvider.classNameFilter(/(action-module)/);

			$permissionProvider.suppressUndefinedPermissionWarning(true);
			//ChartJsProvider.setOptions({ colors : [ '#803690', '#00ADF9', '#DCDCDC', '#46BFBD', '#FDB45C', '#949FB1', '#4D5360'] });

			cfpLoadingBarProvider.includeSpinner = false;
			cfpLoadingBarProvider.includeBar = true;
			cfpLoadingBarProvider.parentSelector = '#loading-bar-container';

			$httpProvider.interceptors.push('responseObserver');
			$httpProvider.defaults.ignoreLoadingBar = true;

			app.register = {
				controller: $controllerProvider.register,
				directive: $compileProvider.directive,
				filter: $filterProvider.register,
				factory: $provide.factory,
				service: $provide.service
			};

			var basePath = "templates/jetbackup/app";

			$routeProvider
				.when("/fileManager/:type/:id", { templateUrl: basePath + "/views/fileManager.ptt", data: { permissions: { only: 'canViewFileManage', redirectTo: '/' } } })
				.when("/settings", { templateUrl: basePath + "/views/settings.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/settings/:section", { templateUrl: basePath + "/views/settings.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/queuePriorities", { templateUrl: basePath + "/views/queuePriorities.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/queuePriorityManage", { templateUrl: basePath + "/views/queuePriorityManage.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/queuePriorityManage/:id", { templateUrl: basePath + "/views/queuePriorityManage.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/restoreConditions", { templateUrl: basePath + "/views/restoreConditions.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/restoreConditionManage", { templateUrl: basePath + "/views/restoreConditionManage.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/restoreConditionManage/:id", { templateUrl: basePath + "/views/restoreConditionManage.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/filePermissions", { templateUrl: basePath + "/views/filePermissions.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/filePermissionManage", { templateUrl: basePath + "/views/filePermissionManage.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/filePermissionManage/:id", { templateUrl: basePath + "/views/filePermissionManage.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/permissions", { templateUrl: basePath + "/views/permissions.ptt", data: { permissions: { only: 'canViewPermissions', redirectTo: '/' } } })
				.when("/permissions/:section", { templateUrl: basePath + "/views/permissions.ptt", data: { permissions: { only: 'canViewPermissions', redirectTo: '/' } } })
				.when("/logs", { templateUrl: basePath + "/views/logs.ptt", data: { permissions: { only: 'canViewLogs', redirectTo: '/' } } })
				.when("/queues", { templateUrl: basePath + "/views/queues.ptt", data: { permissions: { only: 'canViewQueue', redirectTo: '/' } } })
				.when("/alerts", { templateUrl: basePath + "/views/alerts.ptt", data: { permissions: { only: 'canViewAlerts', redirectTo: '/' } } })
				.when("/hooks", { templateUrl: basePath + "/views/hooks.ptt", data: { permissions: { only: 'canViewHooks', redirectTo: '/' } } })
				.when("/hookManage", { templateUrl: basePath + "/views/hookManage.ptt", data: { permissions: { only: 'canManageHooks', redirectTo: '/' } } })
				.when("/hookManage/:id", { templateUrl: basePath + "/views/hookManage.ptt", data: { permissions: { only: 'canManageHooks', redirectTo: '/' } } })
				.when("/backupManager", { templateUrl: basePath + "/views/backupManager.ptt", data: { permissions: { only: 'canViewAnyBackups', redirectTo: '/' } } })
				.when("/backupManager/:section", { templateUrl: basePath + "/views/backupManager.ptt", data: { permissions: { only: 'canViewAnyBackups', redirectTo: '/' } } })
				.when("/cloneJobManage/:id", { templateUrl: basePath + "/views/cloneJobManage.ptt", data: { permissions: { only: 'canManageCloneJobs', redirectTo: '/' } } })
				.when("/cloneJobManage", { templateUrl: basePath + "/views/cloneJobManage.ptt", data: { permissions: { only: 'canManageCloneJobs', redirectTo: '/' } } })
				.when("/cloneJobs", { templateUrl: basePath + "/views/cloneJobs.ptt", data: { permissions: { only: 'canViewCloneJobs', redirectTo: '/' } } })
				.when("/backupJobManage/:id", { templateUrl: basePath + "/views/backupJobManage.ptt", data: { permissions: { only: 'canManageBackupJobs', redirectTo: '/' } } })
				.when("/backupJobManage", { templateUrl: basePath + "/views/backupJobManage.ptt", data: { permissions: { only: 'canManageBackupJobs', redirectTo: '/' } } })
				.when("/backupJobs", { templateUrl: basePath + "/views/backupJobs.ptt", data: { permissions: { only: 'canViewBackupJobs', redirectTo: '/' } } })
				//.when("/scheduleManage/:id", { templateUrl: basePath + "/views/scheduleManage.ptt", data: { permissions: { only: 'canManageSchedules', redirectTo: '/' } } })
				//.when("/scheduleManage", { templateUrl: basePath + "/views/scheduleManage.ptt", data: { permissions: { only: 'canManageSchedules', redirectTo: '/' } } })
				//.when("/schedules", { templateUrl: basePath + "/views/schedules.ptt", data: { permissions: { only: 'canViewSchedules', redirectTo: '/' } } })
				.when("/destinationManage/:id", { templateUrl: basePath + "/views/destinationManage.ptt", data: { permissions: { only: 'canManageDestinations', redirectTo: '/' } } })
				.when("/destinationManage", { templateUrl: basePath + "/views/destinationManage.ptt", data: { permissions: { only: 'canManageDestinations', redirectTo: '/' } } })
				.when("/destinations", { templateUrl: basePath + "/views/destinations.ptt", data: { permissions: { only: 'canViewDestinations', redirectTo: '/' } } })
				//.when("/accountFilterGroups", { templateUrl: basePath + "/views/accountFilterGroups.ptt", data: { permissions: { only: 'canViewAccountFilters', redirectTo: '/' } } })
				//.when("/accountFilterGroupManage", { templateUrl: basePath + "/views/accountFilterGroupManage.ptt", data: { permissions: { only: 'canManageAccountFilters', redirectTo: '/' } } })
				//.when("/accountFilterGroupManage/:id", { templateUrl: basePath + "/views/accountFilterGroupManage.ptt", data: { permissions: { only: 'canManageAccountFilters', redirectTo: '/' } } })
				.when("/accountFilters", { templateUrl: basePath + "/views/accountFilters.ptt", data: { permissions: { only: 'canViewAccountFilters', redirectTo: '/' } } })
				.when("/accountFilterManage", { templateUrl: basePath + "/views/accountFilterManage.ptt", data: { permissions: { only: 'canManageAccountFilters', redirectTo: '/' } } })
				.when("/accountFilterManage/:id", { templateUrl: basePath + "/views/accountFilterManage.ptt", data: { permissions: { only: 'canManageAccountFilters', redirectTo: '/' } } })
				.when("/accounts", { templateUrl: basePath + "/views/accounts.ptt", data: { permissions: { only: 'canViewAccounts', redirectTo: '/' } } })
				.when("/accountManage/:account", { templateUrl: basePath + "/views/accountManage.ptt", data: { permissions: { only: 'canManageAccounts', redirectTo: '/' } } })
				.when("/security", { templateUrl: basePath + "/views/security.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when('/license', { templateUrl: basePath + "/views/license.ptt", data: { permissions: { only: 'isLicenseIssue', redirectTo: '/' } } })
				.when('/agreement', { templateUrl: basePath + "/views/agreement.ptt", data: { permissions: { only: 'isAgreement', except: 'isLicenseIssue', redirectTo: '/' } } })
				.when("/showcase", { templateUrl: basePath + "/views/showcase.ptt", data: { permissions: { only: 'isShowcase', except: ['isLicenseIssue', 'isAgreement', 'isDisasterRecovery'], redirectTo: '/' } } })
				.when("/disasterRecovery", { templateUrl: basePath + "/views/disasterRecovery.ptt", data: { permissions: { only: 'isDisasterRecovery', except: ['isLicenseIssue', 'isAgreement'], redirectTo: '/' } } })
				// .when("/wizard", { templateUrl: basePath + "/views/wizard/restore.ptt" })
				.when("/error", { templateUrl: basePath + "/views/error.ptt" })
				.when("/404", { templateUrl: basePath + "/views/404.ptt" })
				.when('/', { templateUrl: basePath + "/views/dashboard.ptt", data: { permissions: { except: ['isLicenseIssue', 'isAgreement', 'isDisasterRecovery', 'isShowcase'], redirectTo: { isLicenseIssue: '/license', isAgreement: '/agreement', isDisasterRecovery: '/disasterRecovery', isShowcase: '/showcase' } } } })
				.otherwise('/404');
		}
	]).controller("baseController",
		["$rootScope", "$scope", "$route", "$http", "$location", "$timeout", "SweetAlert", "loader", "meta", "lang", "jetapi", "storage", "wizards",
			function($rootScope, $scope, $route, $http, $location, $timeout, SweetAlert, loader, meta, lang, jetapi, storage, wizards) {

				var clear = storage.create('clear');

				if(!clear.isSet('version') || clear.get('version') !== 'v1') {

					storage.deleteAll();

					clear = storage.create('clear');
					clear.set('version', 'v1');
					clear.save();
				}

				loader.setScope($scope);
				loader.setTimeout(200);
				loader.init();

				$rootScope.cp_security_token = window.PAGE.cp_token;
				//$scope.cp_security_token = window.PAGE.cp_token;

				$rootScope.LANG = lang;
				$rootScope.UTILS = UTILS;
				$rootScope.PERM = PERMISSIONS;
				$rootScope.CONST = CONSTS;
				$rootScope.wizards = wizards;
				$rootScope.primaryURL = CONSTS.BASE_URL + window.location.search + '#!';

				$scope.licenseText = PERMISSIONS._licenseText;

				$scope.issueText = '';
				$scope.info = window.PAGE.info;
				$scope.info.version = window.PAGE.system.version;
				$scope.info.tier = window.PAGE.system.tier;
				$scope.total_alerts = window.PAGE.alerts.total;

				$scope.currYear = new Date().getUTCFullYear();

				$scope.menuItem = 'Dashboard';
				$rootScope.$on('menuItem', function(event, item) { $scope.menuItem = item; });

				$scope.expandedRow = undefined;
				$scope.openActions = function(row) {
					$scope.expandedRow = ($scope.expandedRow !== undefined && $scope.expandedRow === row) ? undefined : row;
				};

				$scope.copyToClipboard = function (text) {
					var copyElement = document.createElement("textarea");
					copyElement.style.position = 'fixed';
					copyElement.style.opacity = '0';
					copyElement.textContent = text;
					var body = document.getElementsByTagName('body')[0];
					body.appendChild(copyElement);
					copyElement.select();
					document.execCommand('copy');
					body.removeChild(copyElement);
					SweetAlert.success(lang.t("Success!"), lang.t("Shell command copied to your clipboard"));
				};

				$scope.collapseExpandMenuStatus = localStorage.getItem('menuState');
				if($scope.collapseExpandMenuStatus === null) $scope.collapseExpandMenuStatus = 'collapse';

				$scope.collapseExpandMenu = function() {
					$scope.collapseExpandMenuStatus = $scope.collapseExpandMenuStatus === 'collapse' ? 'expand' : 'collapse';
					localStorage.setItem('menuState', $scope.collapseExpandMenuStatus);
				};

				$scope.changeView = function(view){
					$location.path(view);
					return false;
				};

				$scope.changeViewSorted = function(view, storage_id, field, dir) {
					meta.new(storage_id);
					meta.setSortBy(field);
					meta.setSortDirection(dir);
					$scope.changeView(view);
				};

				$scope.$on('$routeChangeStart', function() { lang.initNS('dashboard'); });

			}
		]
	).factory('responseObserver',["$q", "$window", function responseObserver($q, $window) {
		return {
			'responseError': function(errorResponse) {
				switch (errorResponse.status) {
					case 403:
					case 500:
						location.reload();
					break;
				}
				return $q.reject(errorResponse);
			}
		};
	}]);

	return app;
});