'use strict';

define([
	'app',
	'libraries/consts'
	], function(app, CONSTS) {
	app.controller("accountFilterGroupManage",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $routeParams, $location, $timeout, jetapi, meta, utils, SweetAlert) {
				$rootScope.$emit('menuItem', 'AccountFilters');

				$scope.details = {
					name: '',
					filters: []
				};

				$scope.saveData = utils.duplicateObject($scope.details);

				$scope.filters = {};

				$scope.saveing = false;
				$scope.status = undefined;
				$scope.changed = false;

				$scope.types = {};
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_ACCOUNT] = $scope.LANG.t("Accounts Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_RESELLER] = $scope.LANG.t("Resellers Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_SUSPENSION] = $scope.LANG.t("Suspension Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_DISK_USAGE] = $scope.LANG.t("Disk Space Usage Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_INODE_USAGE] = $scope.LANG.t("Inodes Usage Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_PACKAGE] = $scope.LANG.t("Packages Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_RANGE] = $scope.LANG.t("Characters Range Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_REGEX] = $scope.LANG.t("Regular Expression Filter");

				$scope.toggleFilter = function (filter) {
					var idx = $scope.saveData.filters.indexOf(filter._id);
					if (idx > -1) $scope.saveData.filters.splice(idx, 1);
					else $scope.saveData.filters.push(filter._id);
				};

				$scope.fetchGroupData = function(group_id) {
					jetapi.getAccountFilterGroup({ '_id': group_id }, function(response) {
						$scope.details = response.data;
						$scope.saveData = utils.duplicateObject($scope.details);
					});
				};

				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details, ['_id']);
				}, true);

				$scope.$on('createFilter', function (event) {
					$scope.saveChanges(true, function(response) {
						$scope.$emit('filterResponse', response);
					});
				});

				$scope.$on('$destroy', function() {
					if($scope.changed && !$scope.cancelled)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/accountFilterGroups');
				};

				$scope.saveChanges = function(apply, callback) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details);
					apiParams.action = $scope.details._id ? 'modify' : 'create';

					jetapi.manageAccountFilterGroup(apiParams, function(response) {

						$scope.saveing = false;

						if(callback !== undefined) {
							callback(response);
							return;
						}

						if(response.success)
						{
							$scope.changed = false;
							$scope.saveData._id = response.data._id;
							$scope.details = utils.duplicateObject($scope.saveData);
							if(apply) $location.path('/accountFilterGroupManage/' + response.data._id);
							else $location.path('/accountFilterGroups');

							SweetAlert.swal($scope.LANG.t("Success!"), response.message, "success");
						}
						else
						{
							$scope.status = {
								message: response.message,
								type: "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};

				$scope.fetchFilters = function (callback) {

					$scope.filters = {};
					jetapi.listAccountFilters({}, function (response) {
						for(var i = 0; i < response.data.filters.length; i++) {
							$scope.filters[response.data.filters[i]._id] = response.data.filters[i];
						}
						if(callback !== undefined) callback();
					});
				};

				$timeout($scope.fetchFilters(function() {
					var group_id = $routeParams.id;
					if(group_id) $timeout($scope.fetchGroupData(group_id));
				}));
			}
		]
	);
});