'use strict';

define([
	'app',
	'libraries/consts'
	], function(app, CONSTS) {
	app.controller("accountFilters",
		["$rootScope", "$scope", "$location", "$timeout", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, jetapi, meta, SweetAlert) {
				$rootScope.$emit('menuItem', 'AccountFilters');

				$scope.filters = [];
				$scope.groups = {};
				$scope.loadingFilters = false;
				$scope.saveing = false;
				$scope.status = undefined;
				$scope.filter = 0;

				$scope.filterOptions = [
					{ label: $scope.LANG.t("All Filter Types"), value: 0 },
					{ label: $scope.LANG.t("Accounts Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_ACCOUNT },
					{ label: $scope.LANG.t("Resellers Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_RESELLER },
					{ label: $scope.LANG.t("Suspension Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_SUSPENSION },
					{ label: $scope.LANG.t("Disk Space Usage Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_DISK_USAGE },
					{ label: $scope.LANG.t("Inodes Usage Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_INODE_USAGE },
					{ label: $scope.LANG.t("Packages Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_PACKAGE },
					{ label: $scope.LANG.t("Characters Range Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_RANGE },
					{ label: $scope.LANG.t("Regular Expression Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_REGEX }
				];

				$scope.types = {};
				for(var i = 0; i < $scope.filterOptions.length; i++) if($scope.filterOptions[i].value > 0) $scope.types[$scope.filterOptions[i].value] = $scope.filterOptions[i].label;


				$scope.loaders = {
					delete: false
				};

				meta = meta.new("filters");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("name");
				if(!meta.getSortDirection()) meta.setSortDirection("desc");
				meta.setSortFields(["name","type","owner"]);
				meta.setTotalItems($scope.filters.length);

				$scope.onClickDelete = function(filter) {

					if($scope.saveing) return;

					if(filter.groups_count > 0)
					{
						SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("This filter has an assigned filter groups, Please remove those groups before deleting this filter."));
						return false;
					}

					$scope.saveing = true;
					$scope.loaders.delete = true;

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("This filter will be permanently deleted!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							jetapi.deleteAccountFilter({ _id: filter._id }, function(response) {

								$scope.saveing = false;
								$scope.loaders.delete = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
								}
								else
								{
									SweetAlert.error($scope.LANG.t("Failed!"), response.message);
								}

							});
						}
						else
						{
							$scope.saveing = false;
							$scope.loaders.delete = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Filter delete cancelled"));
						}

					});
				};

				$scope.listGroups = function(callback) {

					jetapi.listAccountFilterGroups({}, function(response) {

						if(!response.success) {
							if(callback !== undefined) callback();
							return;
						}

						for(var i = 0; i < response.data.groups.length; i++) {
							if($scope.groups[response.data.groups[i]._id] === undefined) $scope.groups[response.data.groups[i]._id] = [];
							$scope.groups[response.data.groups[i]._id].push(response.data.groups[i]);
						}

						callback();
					});
				};

				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingFilters = true;
					$scope.status = undefined;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: {},
						filter: meta.getFilter()
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();
					if($scope.filter) apiParams.find.type = $scope.filter;

					$scope.filters = [];

					jetapi.listAccountFilters(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.filters);
						$scope.filters = response.data.filters;
						for(var i = 0; i < $scope.filters.length; i++) $scope.groups[$scope.filters[i]._id] = [];
						$scope.listGroups(function () { $scope.loadingFilters = false; });
					});
				};

				$timeout($scope.fetchData);
			}
		]
	);
});