'use strict';

define([
	'app',
	'libraries/consts'
	], function(app, CONSTS) {
	app.controller("accountManage",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $routeParams, $location, $timeout, jetapi, meta, utils, SweetAlert) {
				$rootScope.$emit('menuItem', 'Accounts');

				if(!$routeParams.account) $location.path("/");

				$scope.details = {
					email: '',
					maxsnapshots: '',
					queue_priority: ''
				};

				$scope.saveData = utils.duplicateObject($scope.details);

				$scope.saveing = false;
				$scope.status = undefined;
				$scope.changed = false;
				$scope.cancelled = false;

				$scope.queuePriorities = [
						{ _id: '', name: $scope.LANG.t('Default') }
				];

				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details, ['_id']);
				}, true);

				$scope.$on('$destroy', function() {
					if($scope.changed && !$scope.cancelled)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.resetEncryptionKey = function() {

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("Once you reset the account encryption key, ALL BACKUPS becomes inaccessible and will be deleted."),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), $scope.LANG.t("Delete Encryption Key")]
					}, function(isConfirm){
						if (isConfirm) {

							jetapi.resetEncryptionKey({ username: $scope.details.username }, function (response) {

								if(!response.success) {
									$scope.status = {
										message: response.message,
										type: "danger",
										closeable: false,
										ttl: 10000
									};
									return;
								}

								$scope.details.secret_key = false;
								SweetAlert.swal($scope.LANG.t("Success!"), response.message, "success");
							});
						}
					});
				};

				$scope.fetchQueuePriorities = function() {

					jetapi.listQueuePriorities({}, function (response) {
						if(!response.success) {
							$scope.status = {
								message: response.message,
								type: "danger",
								closeable: false,
								ttl: 10000
							};
							return;
						}
						var priorities = response.data.priorities;
						for(var i=0; i< priorities.length; i++ ){
							$scope.queuePriorities.push(priorities[i]);
						}
					});
				};

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/accounts');
				};

				$scope.saveChanges = function(apply) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details);
					apiParams.username = $scope.details.username;

					jetapi.manageAccount(apiParams, function(response) {

						$scope.saveing = false;

						if(response.success)
						{
							$scope.changed = false;
							$scope.details.email = $scope.saveData.email;
							$scope.details.maxsnapshots = $scope.saveData.maxsnapshots;
							//$location.path('/accountManage/' + $scope.details.username);
							if(!apply) $location.path('/accounts');
							SweetAlert.swal($scope.LANG.t("Success!"), response.message, "success");
						}
						else
						{
							$scope.status = {
								message: response.message,
								type: "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};

				$scope.fetchAccountData = function(account) {
					jetapi.getAccount({ '_id': account }, function(response) {
						$scope.details = response.data;
						$scope.saveData.email = $scope.details.email;
						$scope.saveData.maxsnapshots = $scope.details.maxsnapshots;
						$scope.saveData.queue_priority = $scope.details.queue_priority ? $scope.details.queue_priority : '';
					});
				};

				$timeout(function () {
					$scope.fetchQueuePriorities();
					$scope.fetchAccountData($routeParams.account);
				});
			}
		]
	);
});