'use strict';

define([
	'app',
	'libraries/permissions',
	'libraries/consts'
], function(app, PERMISSIONS, CONSTS) {
	app.controller("backupJobManage",
		["$rootScope", "$scope", "$routeParams", "$templateCache", "$route", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $templateCache, $route, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert) {
				$rootScope.$emit('menuItem', 'BackupJobs');

				$scope.cancelled = false;

				$scope.saveData.backuptype = CONSTS.BACKUP_TYPE_FULL;
				$scope.saveData.backuptypes = CONSTS.BACKUP_TYPE_FULL;

				if($scope.jobWizard ) {
					if($scope.wizardData && $scope.wizardData.destinationId)
						$scope.saveData.destination = $scope.wizardData.destinationId;
				}


				$scope.flags = [
					{ label: $scope.LANG.t("Incremental"), value: 1 },
					{ label: $scope.LANG.t("Archived"), value: 2 },
					{ label: $scope.LANG.t("Compressed"), value: 4 }
				];

				$scope.types = [
					{ label: $scope.LANG.t("Accounts"), value: CONSTS.BACKUP_TYPE_FULL },
					{ label: $scope.LANG.t("Directories"), value: CONSTS.BACKUP_TYPE_DIRECTORIES },
					{ label: $scope.LANG.t("Replicate"), value: CONSTS.BACKUP_TYPE_REPLICATE }
				];

				$scope.account_types = [
					{ label: $scope.LANG.t("cPanel Configurations"), value: CONSTS.BACKUP_TYPE_CONFIG },
					{ label: $scope.LANG.t("Home dir Files"), value: CONSTS.BACKUP_TYPE_FILES },
					{ label: $scope.LANG.t("Databases (Creates a full database dump)"), value: CONSTS.BACKUP_TYPE_DATABASES },
					{ label: $scope.LANG.t("Emails"), value: CONSTS.BACKUP_TYPE_EMAILS },
					{ label: $scope.LANG.t("Cron Jobs"), value: CONSTS.BACKUP_TYPE_CRON_JOBS },
					{ label: $scope.LANG.t("DNS Zones"), value: CONSTS.BACKUP_TYPE_DNS_ZONES },
					{ label: $scope.LANG.t("SSL Certificates"), value: CONSTS.BACKUP_TYPE_SSL_CERTIFICATES }
				];


				$scope.limit_accounts_rates = [
					{ label: $scope.LANG.t("Unlimited"), value: 0 },
					{ label: 1, value: 1 },
					{ label: 2, value: 2 },
					{ label: 3, value: 3 },
					{ label: 5, value: 5 },
					{ label: 10, value: 10 }
				];

				$scope.runtime_ttl_rates = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
					{ label: 60, value: 60 },
					{ label: 120, value: 120 },
					{ label: 300, value: 300 },
					{ label: 600, value: 600 }
				];

				$scope.monitor_runfor_ttl_rates = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
					{ label: 1, value: 1 },
					{ label: 2, value: 2 },
					{ label: 3, value: 3 },
					{ label: 5, value: 5 },
					{ label: 7, value: 7 }
				];

				$scope.monitor_notran_ttl_rates = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
					{ label: 1, value: 1 },
					{ label: 2, value: 2 },
					{ label: 3, value: 3 },
					{ label: 5, value: 5 },
					{ label: 7, value: 7 }
				];

				$scope.load_pause_rates = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
				];

				$scope.load_rates = [];
				for (var i=1; i<=32; i++) $scope.load_rates.push(i);

				$scope.retry_times_rates = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
					{ label: 1, value: 1 },
					{ label: 2, value: 2 },
					{ label: 3, value: 3 },
					{ label: 4, value: 4 },
					{ label: 5, value: 5 }
				];


				if($scope.jobWizard && !$scope.wizardData.jobId){
					$scope.saveData.name = 'JetBackup_' + parseInt(new Date().getTime()%10000);
				}


				$scope.typeDisabled = function(type) {

					if($scope.destinationsIds[$scope.saveData.destination] !== undefined)
					{
						var destType = $scope.destinationsIds[$scope.saveData.destination].type;

						if(type === 512 && (destType === 'Dropbox' || destType === 'GoogleDrive' || destType === 'Backblaze')) {
							if($scope.saveData.backuptypes === 512) {
								$scope.saveData.backuptypes = 127;
								$scope.saveData.backuptype = 127;
							}
							return true;
						}
					}
				};

				$scope.flagDisabled = function(flag) {
					if(flag > 1 && $scope.saveData.backuptypes === 512) {
						$scope.saveData.flag = 1;
						return true;
					}

					if($scope.destinationsIds[$scope.saveData.destination] !== undefined)
					{
						var destType = $scope.destinationsIds[$scope.saveData.destination].type;

						if(flag === 1 && (destType === 'Dropbox' || destType === 'GoogleDrive' || destType === 'Backblaze')) {
							if($scope.saveData.flag == 1) $scope.saveData.flag = 2;
							return true;
						}
					}
				};

				var checkBackupTypes = function() {
					if($scope.saveData.backuptype > 127) return;

					if($scope.saveData.flag == 1){
						// enable all types
						$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_CONFIG] = false;
						$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_FILES] = false;
						$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_DATABASES] = false;
						$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_EMAILS] = false;
						$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_CRON_JOBS] = false;
						$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_DNS_ZONES] = false;
						$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_SSL_CERTIFICATES] = false;
						return;
					}
					$scope.saveData.backuptypes = 127;
					// disable all types
					$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_CONFIG] = true;
					$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_FILES] = true;
					$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_DATABASES] = true;
					$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_EMAILS] = true;
					$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_CRON_JOBS] = true;
					$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_DNS_ZONES] = true;
					$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_SSL_CERTIFICATES] = true;
				};

				$scope.$watch('saveData.flag', function() {
					checkBackupTypes();
				});

				$scope.$watch('saveData.backuptype', function() {
					checkBackupTypes();
				});

				$scope.toggleBackupType = function(type) {
					$scope.saveData.backuptypes = type;
				};

				$scope.backupTypesDisabled = {};
				$scope.toggleAccountBackupTypes = function(type) {

					// disable SSL Certificate backup if config backup is disabled
					// TODO remove this after implementing SSL backup
					if(type === CONSTS.BACKUP_TYPE_CONFIG) {
						$scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_SSL_CERTIFICATES] = $scope.typeChecked(type, $scope.saveData.backuptypes);
						if($scope.backupTypesDisabled[CONSTS.BACKUP_TYPE_SSL_CERTIFICATES] && $scope.typeChecked(CONSTS.BACKUP_TYPE_SSL_CERTIFICATES, $scope.saveData.backuptypes)) {
							$scope.toggleAccountBackupTypes(CONSTS.BACKUP_TYPE_SSL_CERTIFICATES);
						}
					}

					if($scope.typeChecked(type, $scope.saveData.backuptypes))
					{
						$scope.saveData.backuptypes ^= type;
					}
					else
					{
						$scope.saveData.backuptypes |= type;
					}
				};

				var checkDuplicateIncludeItems = function(value, list) {
					var withoutTrailing = value.replace(/\/+$/, '');
					var withTrailing = withoutTrailing + '/';
					return (list.indexOf(withoutTrailing) >= 0 || list.indexOf(withTrailing) >= 0);
				};

				$scope.addListRow = function (value, list) {
					if($scope[value] === undefined || !$scope[value].trim()) return;

					if(checkDuplicateIncludeItems($scope[value].trim(), list)){
						SweetAlert.error($scope.LANG.t("Error!"), $scope.LANG.t('The provided path ("%s") already exists on the list.', $scope[value].trim()));
					}
					else if($scope.validateListPath($scope[value].trim(), value === 'excluderow')) {
						list.push($scope[value].trim());
					}
					else {
						SweetAlert.error($scope.LANG.t("Error!"), $scope.LANG.t('The provided path ("%s") is invalid. The path must start with a "/"' + (value !== 'excluderow' ? ', also patterns are not allowed.' : '.'), $scope[value].trim() ));
					}

					$scope[value] = '';

				};

				$scope.addMultiListRow = function(type, list) {

					if(type == 'include') {
						$scope.listTitle = $scope.LANG.t("Directories and Files to include");
						$scope.listData = $scope.saveData.includelist.join("\n");
					} else {
						$scope.listTitle = $scope.LANG.t("Directories and Files to exclude");
						$scope.listData = $scope.saveData.excludelist.join("\n");
					}

					$scope.listUIB = $uibModal.open({
						animation: true,
						scope: $scope,
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/listSelection.ptt'
					});

					$scope.listUIB.result.then(function(records) {
						while(list.length) list.pop();
						records = records.split("\n");
						var invalidPaths = [];
						var duplicatePaths = [];
						for(var i = 0; i < records.length; i++) {
							if(type === 'include' && checkDuplicateIncludeItems(records[i], list)) duplicatePaths.push(records[i]);
							else if($scope.validateListPath(records[i], type === 'exclude')) {
								if (list.indexOf(records[i]) < 0 ) list.push(records[i]);
							}
							else invalidPaths.push(records[i]);
						}

						if(invalidPaths.length) SweetAlert.error($scope.LANG.t("Error!"), $scope.LANG.t('The following paths ("%s") is invalid. The path must start with a "/" and can\'t have trailing "/"' + (type === 'include' ? ', also patterns are not allowed.' : '.'), invalidPaths.join(", ") ))
						if(duplicatePaths.length) SweetAlert.error($scope.LANG.t("Error!"), $scope.LANG.t('The provided path(s) ("%s") already exists on the list.', duplicatePaths.join(", ")));
					});
				};

				$scope.$on('validateDestination', function(event, data) {

					if(data.destination.clone_destination || data.destination.engine != CONSTS.DESTINATION_ENGINE_JETBACKUP) {
						data.callback();
						return;
					}

					data.callback(data.destination);
				});

				$scope.$on('fetchJobData', function(event, data) {
					$scope.fetchJobData(data._id, data.callback);
				});

				$scope.fetchJobData = function(job_id, callback) {

					jetapi.getBackupJob({ '_id': job_id }, function(response) {
						if(!response.success) return;
						$scope.details = response.data;

						$scope.details.backuptype = $scope.details.backuptypes;
						if($scope.details.backuptype < 127) $scope.details.backuptype = 127;

						callback($scope.details);
					});

				};

				$scope.$on('$destroy', function() {

					if($scope.changed && !$scope.cancelled && !$scope.jobWizard)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/backupJobs');
				};

				$scope.saveChanges = function(apply, callback) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details, ['backuptype']);

					if(apiParams.time_parsed) apiParams.time = $scope.parseTime(apiParams.time_parsed);
					apiParams.action = $scope.details._id ? 'modify' : 'create';

					$scope.validateSchedules($scope.saveData.schedules, function(isConfirm) {
						if(isConfirm) {
							jetapi.manageBackupJob(apiParams, function(response) {

								$scope.saveing = false;
								if(callback !== undefined && typeof callback === 'function') callback(response);

								if(response.success)
								{
									$scope.details = utils.duplicateObject($scope.saveData);
									$scope.updateData($scope.details);

									if(callback === undefined) {
										if (apply) $location.path('/backupJobManage/' + response.data._id);
										else $location.path('/backupJobs');
										SweetAlert.swal($scope.LANG.t("Success!"), response.message, "success");
									}
								}
								else
								{
									$scope.status = {
										message: response.message,
										type: response.success ? "success" : "danger",
										closeable: false,
										ttl: 10000
									};
								}
							});
						} else $scope.saveing = false;
					});
				};

				$scope.hideTime = function() {
					if(!$scope.saveData.schedules || !$scope.saveData.schedules.length || $scope.jobWizard ) return true;
					for(var i=0; i < $scope.saveData.schedules.length; i++) {
						if ($scope.schedules[$scope.saveData.schedules[i]._id].type < 5) return false;
					}
					return true;
				};



				//Handle backupJobWizard broadcast
				$scope.$on($scope.wizardName, function (env, data) {
					$scope.saveData.destination = data.destinationId;
					$scope.saveData.disabled = 1;
					$scope.saveChanges(false, function (response) {
						$scope.process(response);
						// $scope.$emit("onWizardResponse", response);
					});
				});


			}
		]
	);

});