'use strict';

define([
	'app',
	'libraries/utils',
	'libraries/permissions'
	], function(app, UTILS, PERMISSIONS) {
	app.controller("backupManager",
		["$rootScope", "$scope", "$routeParams", "$location", "$uibModal", "jetapi",
			function ($rootScope, $scope, $routeParams, $location, $uibModal, jetapi) {
				$rootScope.$emit('menuItem', 'BackupManager');

				//$scope.currentSection = {};
				$scope.config = {};

				$scope.sections = [];
				var sections = [
					{ _id: 'accounts', name: $scope.LANG.t('Accounts'), template: 'backupManagerAccounts.ptt', icon: 'fa-users', permission: PERMISSIONS.canViewAccountsBackups },
					{ _id: 'directories', name: $scope.LANG.t('Directories'), template: 'backupManagerDirectories.ptt', icon: 'fa-folder', permission: PERMISSIONS.canViewDirectoriesBackups }
				];

				for(var i = 0; i < sections.length; i++) {
					if(sections[i].permission) $scope.sections.push(sections[i]);
				}

				$scope.getSection = function (section_id) {
					for(var i = 0; i < $scope.sections.length; i++) {
						if($scope.sections[i]._id === section_id) {
							return $scope.sections[i];
						}
					}

					return $scope.sections[0];
				};

				$scope.changeSection = function(section) {
					$location.path('/backupManager/' + section._id);
				};

				$scope.loadSection = function (section) {
					$scope.currentSection = section;
					$scope.backupSection = 'templates/jetbackup/app/views/' + section.template;
				};

				jetapi.getConfigurations({}, function(response) {
					if(!response.success) return;
					$scope.config = response.data;
				});

				$scope.lockTimeSelection = function (callback, skip) {

					if(skip && callback !== undefined && typeof callback == 'function') {
						callback(0);
						return;
					}

					var modalInstance = $uibModal.open({
						animation: true,
						size: "md",
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/snapshotLockSelection.ptt',
						controller: 'snapshotLockSelection',
						controllerAs: '$scope',
						resolve: {
							filters: function() { return $scope.filters; } ,
							details: function() { return $scope.details; }
						}
					});

					modalInstance.result.then(function(ttl) {
						if(callback !== undefined && typeof callback == 'function') callback(ttl);
					}, function () {
						//$log.info('Modal dismissed at: ' + new Date());
					});
				};

				$scope.loadSection($scope.getSection($routeParams.section ? $routeParams.section : 'accounts'));
			}]);
});