'use strict';

define([
	'app',
	], function(app) {
	app.controller("cloneJobManage",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert) {
				$rootScope.$emit('menuItem', 'CloneJobs');

				$scope.cancelled = false;

				$scope.iptypes = [
					{ label: $scope.LANG.t("Restore the account with shared IP"), value: 1 },
					{ label: $scope.LANG.t("Try to match dedicated IP status (Only if possible)"), value: 2 }
				];

				$scope.runtime_ttl_rates = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
					{ label: 60, value: 60 },
					{ label: 120, value: 120 },
					{ label: 300, value: 300 },
					{ label: 600, value: 600 }
				];

				$scope.monitor_runfor_ttl_rates = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
					{ label: 1, value: 1 },
					{ label: 2, value: 2 },
					{ label: 3, value: 3 },
					{ label: 5, value: 5 },
					{ label: 7, value: 7 }
				];

				$scope.monitor_notran_ttl_rates = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
					{ label: 1, value: 1 },
					{ label: 2, value: 2 },
					{ label: 3, value: 3 },
					{ label: 5, value: 5 },
					{ label: 7, value: 7 }
				];

				$scope.addListRow = function (value, list) {
					if($scope[value] === undefined || !$scope[value].trim()) return;
					if($scope.validateListPath($scope[value].trim(), value === 'excluderow')) list.push($scope[value].trim());
					else SweetAlert.error($scope.LANG.t("Error!"), $scope.LANG.t("The provided path (\"%s\") is invalid. The path must start with a \"/\"" + (value !== 'excluderow' ? ", also patterns are not allowed." : '.'), $scope[value].trim()));
					$scope[value] = '';
				};

				$scope.addMultiListRow = function(list) {

					$scope.listTitle = $scope.LANG.t("Directories and Files to excludes");
					$scope.listData = $scope.saveData.excludelist.join("\n");

					$scope.listUIB = $uibModal.open({
						animation: true,
						scope: $scope,
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/listSelection.ptt'
					});

					$scope.listUIB.result.then(function(records) {
						while(list.length) list.pop();
						records = records.split("\n");
						var invalidPaths = [];
						for(var i = 0; i < records.length; i++) {
							if($scope.validateListPath(records[i].trim(), true)) list.push(records[i].trim());
							else invalidPaths.push(records[i].trim());
						}
						if(invalidPaths.length) SweetAlert.error($scope.LANG.t("Error!"), $scope.LANG.t("The following paths (\"%s\") is invalid. The path must start with a \"/\" and can't have trailing \"/\".", invalidPaths.join(", ")))
					});
				};


				$scope.$on('validateDestination', function(event, data) {
					if(data.destination.backup_destination) data.callback();
					else data.callback(data.destination);
				});

				$scope.$on('fetchJobData', function(event, data) {
					$scope.fetchJobData(data._id, data.callback);
				});

				$scope.fetchJobData = function(job_id, callback) {

					jetapi.getCloneJob({ '_id': job_id }, function(response) {
						$scope.details = response.data;
						callback($scope.details);
					});

				};

				$scope.$on('$destroy', function() {
					if($scope.changed && !$scope.cancelled)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) $scope.saveChanges();
						});
					}
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/cloneJobs');
				};

				$scope.saveChanges = function(apply) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details);

					if(apiParams.time_parsed) apiParams.time = $scope.parseTime(apiParams.time_parsed);
					apiParams.action = $scope.details._id ? 'modify' : 'create';

					$scope.validateSchedules($scope.saveData.schedules, function (isConfirm) {
						if(isConfirm) {
							jetapi.manageCloneJob(apiParams, function(response) {

								$scope.saveing = false;

								if(response.success)
								{
									$scope.details = utils.duplicateObject($scope.saveData);
									$scope.updateData($scope.details);
									if(apply) $location.path('/cloneJobManage/' + response.data._id);
									else $location.path('/cloneJobs');
									SweetAlert.swal($scope.LANG.t("Success!"), response.message, "success");
								}
								else
								{
									$scope.status = {
										message: response.message,
										type: response.success ? "success" : "danger",
										closeable: false,
										ttl: 10000
									};
								}
							});
						} else $scope.saveing = false;
					});
				};
			}
		]
	);
});