'use strict';

define([
	'app',
	'libraries/permissions',
	'libraries/consts'
	], function(app, PERMISSIONS, CONSTS) {
	app.controller("disasterRecoveryDestination",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $routeParams, $location, $timeout, jetapi, meta, utils, SweetAlert) {

				$scope.destination_id = '';
				$scope.details = {
					type: 'Local',
					path: '/',
					checked: true
				};

				$scope.types = [
					{ label: "Local", value: 'Local', group: $scope.LANG.t('Backup Destinations') },
					{ label: "SSH", value: 'SSH', group: $scope.LANG.t('Backup Destinations') },
					{ label: "SFTP", value: 'SFTP', group: $scope.LANG.t('Backup Destinations') },
					{ label: "FTP", value: 'FTP', group: $scope.LANG.t('Backup Destinations') },
					{ label: "Rsync.net", value: 'RsyncNet', group: $scope.LANG.t('Backup Destinations') },
					{ label: "Amazon S3", value: 'AmazonS3', group: $scope.LANG.t('Backup Destinations') },
					{ label: "Google Drive", value: 'GoogleDrive', group: $scope.LANG.t('Backup Destinations') },
					{ label: "Dropbox", value: 'Dropbox', group: $scope.LANG.t('Backup Destinations') },
					{ label: "Backblaze", value: 'Backblaze', group: $scope.LANG.t('Backup Destinations') }
				];

				$scope.loadDestinationType = function(type, defaults) {

					$scope.saveData = utils.duplicateObject($scope.details);

					if(defaults)
					{
						$scope.saveData.type = type;

						switch(type)
						{
							case 'SSH':
							case 'RsyncNet':
								$scope.saveData.path = './';
								$scope.saveData.port = 22;
								$scope.saveData.timeout = 30;
								break;

							case 'FTP':
							case 'SFTP':
								$scope.saveData.timeout = 30;
								$scope.saveData.retries = 5;
								$scope.saveData.connections = 2;
								$scope.saveData.ssl = false;
								$scope.saveData.passive = true;
								break;

							case 'AmazonS3':
								$scope.saveData.path = '';
								$scope.saveData.concurrent = 10;
								$scope.saveData.bandwidth = 0;
								$scope.saveData.chunksize = 8;
								break;

							case 'GoogleDrive':
								$scope.saveData.path = '';
								$scope.saveData.authtype = 1;
								break;

							case 'Backblaze':
								$scope.saveData.path = '';
								$scope.saveData.retries = 3;
								$scope.saveData.retrytimeout = 1;
								break;

							case 'Dropbox':
								$scope.saveData.path = '';
								break;
						}
					}

					$scope.destinationType = 'templates/jetbackup/app/views/destinations/' + type + '.ptt';
				};

				$scope.$on('nextStep', function() {

					var apiParams = utils.saveParams($scope.saveData, $scope.details, ['checked'], function(key) {
						return (key === 'type');
					});

					apiParams.action = 'create';
					apiParams.name = 'Disaster Recovery ' + Date.now();

					jetapi.manageDestination(apiParams, function(response) {

						if(!response.success) {
							$scope.$emit("error", response.message);
							return;
						}

						$scope.destination_id = response.data._id;

						jetapi.reindex({ destination_id: $scope.destination_id, type: CONSTS.BACKUP_DISASTER_RECOVERY }, function(response) {

							if(!response.success) {
								$scope.$emit("error", response.message);
								return;
							}

							jetapi.listBackups({ destination: $scope.destination_id, type: CONSTS.BACKUP_TYPE_DISASTER_RECOVERY, sort: { created: -1 } }, function (response) {

								if(!response.success) {
									$scope.$emit("error", response.message);
									return;
								}

								if(!response.data.total) {
									$scope.$emit("error", "No JetBackup configurations files found on this destination");
									return;
								}

								$scope.$emit("commitNextStep", {
									destination_id: $scope.destination_id,
									drbackups: response.data.backups,
									drbackup: response.data.backups[0]._id
								});
							});
						});
					});
				});

				$scope.loadDestinationType($scope.details.type, true);

	}]);
});