'use strict';

define([
		'app',
		'libraries/consts',
		'libraries/permissions'
	], function(app, CONSTS, PERMISSIONS) {

		app.controller("fileManager",
			["$rootScope", "$scope", "$interval", "$timeout", "jetapi", "meta", "SweetAlert", "$window", "$routeParams", "$location",
				function($rootScope, $scope, $interval, $timeout, jetapi, meta, SweetAlert, $window, $routeParams, $location) {
					if(($routeParams.type == 'destination' && !PERMISSIONS.canManageDestinations) ||
							($routeParams.type == 'backup') && !PERMISSIONS.canViewFileBackups) {
						$location.path("/");
						return false;
					}
					$rootScope.$emit('menuItem', 'BackupManager');
					$scope.type = $routeParams.type;
					$scope.typeId = $routeParams.id;
					$scope.typeDetails = { name: '' };
					$scope.files = [];
					$scope.loadingFiles = false;
					$scope.currentPath = '';
					$scope.breadcrumbs = [];
					$scope.filesIndex = {};
					$scope.actionStatus = undefined;
					$scope.actionModule = undefined;
					$scope.config = {};
					$scope.backup_type = 2;
					$scope.restoringBackup = false;
					$scope.downloadingBackup = false;
					$scope.downloads = [];
					$scope.restores = [];
					$scope.downloadsTmp = [];
					$scope.queues = [];
					$scope.isDirectories = false;
					$scope.showhidden = false;

					var statusInterval;

					$scope.clearStatus = function() {
						$scope.actionStatus = undefined;
					};

					$scope.cancelAction = function() {
						$scope.actionStatus = undefined;
						$scope.actionModule = undefined;
					};

					$scope.changeView = function(view){
						$location.path(view); // path not hash
					};

					$scope.onClickRestore = function() {
						var selected = getSelected();

						if(!selected.length)
						{
							$scope.actionStatus = {
								message: "Please select files to restore",
								type: "danger",
								closeable: true,
								ttl: 10000
							};
							return;
						}

						$scope.actionModule = ($scope.actionModule === 'restore') ? undefined : 'restore';
					};

					$scope.onClickRestoreConfirm = function() {

						$scope.restoringBackup = true;
						$scope.actionStatus = undefined;

						var apiParams = {};
						apiParams['_id'] = $scope.typeId;
						apiParams['files'] = getSelected();

						jetapi.addQueueRestore(apiParams, function(response) {

							if(!response.success) {
								SweetAlert.error($scope.LANG.t("Error!"), response.message);
								$scope.restoringBackup = false;
								return false;
							}

							$scope.queues.push(response.data._id);

							$scope.restoringBackup = false;
							$scope.actionModule = undefined;
							$scope.uncheckAll();
							$scope.getRestores();

							SweetAlert.success($scope.LANG.t("Success!"), response.message);
							return true;
						});
					};

					$scope.onClickDownload = function() {

						var selected = getSelected();

						if(!selected.length)
						{
							$scope.actionStatus = {
								message: "Please select files to download",
								type: "danger",
								closeable: true,
								ttl: 10000
							};

							return;
						}

						$scope.actionModule = ($scope.actionModule === 'download') ? undefined : 'download';
					};

					$scope.onClickDownloadConfirm = function() {

						$scope.downloadingBackup = true;
						$scope.actionStatus = undefined;

						var apiParams = {};
						apiParams['_id'] = $scope.typeId;
						apiParams['type'] = $scope.type;
						apiParams['files'] = getSelected();

						jetapi.addQueueDownload(apiParams, function(response) {

							if(!response.success) {
								SweetAlert.error($scope.LANG.t("Error!"), response.message);
								$scope.actionStatus = { type: "danger", message: response.message };
								$scope.downloadingBackup = false;
								return false;
							}

							$scope.queues.push(response.data._id);

							$scope.downloadingBackup = false;
							$scope.actionModule = undefined;
							$scope.uncheckAll();
							$scope.getDownloads();

							SweetAlert.success($scope.LANG.t("Success!"), response.message);
							return true;
						});
					};

					$scope.isConditionsAgreed = function() {

						for(var i in $scope.config.restore_conditions)
						{
							if($scope.config.restore_conditions[i].type !== 0 && $scope.config.restore_conditions[i].type !== $scope.backup_type) continue;
							if(!$scope.config.restore_conditions[i].checked) return true
						}

						return false;
					};

					var getSelected = function() {

						var selected = [];

						for(var i in $scope.filesIndex)
						{
							for(var j in $scope.filesIndex[i].data.files)
							{
								if($scope.filesIndex[i].data.files[j] != null && $scope.filesIndex[i].data.files[j].checked !== undefined && $scope.filesIndex[i].data.files[j].checked)
								{
									selected.push($scope.filesIndex[i].data.files[j].path);
								}
							}
						}

						return selected;
					};

					$scope.canPerformAction = function () {

						for(var i in $scope.filesIndex)
						{
							for(var j in $scope.filesIndex[i].data.files)
							{
								if($scope.filesIndex[i].data.files[j] != null && $scope.filesIndex[i].data.files[j].checked !== undefined && $scope.filesIndex[i].data.files[j].checked)
								{
									return false;
								}
							}
						}

						return true;
					};

					$scope.isDisabled = function() {
						var file = this.file.parent;
						while(!file.checked && file.parent) file = file.parent;
						return file.checked;
					};

					$scope.isChecked= function() {
						var file = this.file;
						while(!file.checked && file.parent) file = file.parent;
						return file.checked;
					};

					var uncheckChildes = function(file) {
						file.children = 0;
						for(var i in $scope.filesIndex)	{
							for(var j in $scope.filesIndex[i].data.files) {
								var curr = $scope.filesIndex[i].data.files[j];
								if(curr.path && file.path != curr.path && curr.path.indexOf(file.path) == 0) {
									curr.checked = false;
								}
							}
						}
					};

					$scope.calculateInput = function(value) {

						if(value === undefined)
						{
							var file = this.file;
							while(!file.checked && file.parent) file = file.parent;
							return file.checked;
						}

						this.file.checked = value;
						if (value) uncheckChildes(this.file);

						var file = this.file;

						while(file.parent !== undefined) {
							file = file.parent;

							if(file.children === undefined) file.children = 0;

							if(value) file.children++;
							else file.children--;
						}
					};

					var manageBreadcrumbs = function(file) {
						if(file.path === '/') file.name = '/' + $scope.type + '-root';
						$scope.breadcrumbs.unshift(file);
						if(file.parent !== undefined) manageBreadcrumbs(file.parent);
					};

					var handleResponse = function(response, file) {
						$scope.filesIndex[file.path] = response;
						$scope.files = response.data.files;
						$scope.loadingFiles = false;
					};

					$scope.toggleHidden = function() {
						$scope.showhidden = !$scope.showhidden;
					};

					$scope.fetch = function(file) {

						jetapi.getConfigurations({}, function(response) {
							if(!response.success) return;
							$scope.config = response.data;
						});

						if(file.path === undefined) file.path = '/';

						$scope.loadingFiles = true;
						$scope.files = [];

						$scope.breadcrumbs = [];
						manageBreadcrumbs(file);

						if($scope.filesIndex[file.path] !== undefined)
						{
							handleResponse($scope.filesIndex[file.path], file);
							return;
						}

						var apiParams = {};
						apiParams['_id'] = $scope.typeId;
						apiParams['type'] = $scope.type;
						apiParams['path'] = file.path;

						jetapi.fileManager(apiParams, function(response) {

							if(!response.success) {
								SweetAlert.error($scope.LANG.t("Error!"), response.message);
								return false;
							}

							for(var i = 0; i < response.data.files.length; i++)
							{
								response.data.files[i].path = file.path + (file.path === '/' ? '' : '/') + response.data.files[i].name;
								response.data.files[i].parent = file;
								response.data.files[i].hidden = response.data.files[i].name.substr(0, 1) === '.';
							}

							handleResponse(response, file);
							return true;
						});

					};

					$scope.uncheckAll = function () {

						for(var i in $scope.filesIndex)
						{
							for(var j in $scope.filesIndex[i].data.files)
							{
								if($scope.filesIndex[i].data.files[j] != null )
								{
									$scope.filesIndex[i].data.files[j].checked = false;
									$scope.filesIndex[i].data.files[j].children = 0;

								}
							}
						}

					};

					
					$scope.checkAll = function () {
						$scope.uncheckAll();
						var path = $scope.breadcrumbs[$scope.breadcrumbs.length-1].path;
						for(var i in $scope.filesIndex)	{
							for(var j in $scope.filesIndex[i].data.files) {
								var file = $scope.filesIndex[i].data.files[j];
								if(file.path && path != file.path && !file.path.indexOf(path) && (file.type == 'File' || file.type == 'Directory')){
									$scope.filesIndex[i].data.files[j].checked = true;
									file.parent.children ? file.parent.children++ : file.parent.children = 1;
								}
							}
						}
					};

					$scope.directDownload = function(download) {
						window.location = 'cgi/addons/jetbackup/download.cgi?download_id=' + download.download_id;
					};

					$scope.getDownloads = function () {
						$scope.downloads = [];
						$scope.loadingDownloads = true;

						var apiParams = {};
						apiParams['_id'] = $scope.typeId;
						apiParams['type'] = $scope.type;

						jetapi.getBackupDownloads(apiParams, function(response) {

							if (!response.success) {
								$scope.downloads = [];
								SweetAlert.error($scope.LANG.t("Error!"), response.message);
								$scope.loadingDownloads = false;
								return false;
							}

							for(var i = 0; i < response.data.downloads.length; i++) {
								$scope.downloads.push(response.data.downloads[i]);
								if ($scope.downloads[i].ready) $scope.downloads[i].status = 100;
								else $scope.downloads[i].status = 1;
							}

							var params = { type: CONSTS.QUEUE_TYPE_DOWNLOAD, find: find };
							if($scope.type == 'destination') params.destination_id = $scope.typeId;
							else {
								params.snap_item_id = $scope.typeId;
								params.find = {status:{'$gt':100}};
							}

							jetapi.listQueueItems(params, function(response) {

								if(!response.success) {
									SweetAlert.error($scope.LANG.t("Error!"), response.message);
									$scope.loadingRestores = false;
									return false;
								}

								for(var i = 0; i < response.data.queue.length; i++)
								{
									var data = response.data.queue[i];

									var filename = '';
									if(data.status > 100) filename = $scope.LANG.t("Download Failed");
									else if(data.status < 100) filename = $scope.LANG.t("Download in Progress");

									if(data.status != 100 ){
										$scope.downloads.push({
											queue_id: data._id,
											status: data.status,
											ready: !(parseInt(data.status) > 100 || parseInt(data.status) < 100),
											filename: filename,
											download_id: data.download_id
										});
									}

								}

								$scope.loadingDownloads = false;
								return true;
							});

							return true;
						});
					};

					$scope.getRestores = function () {

						$scope.loadingRestores = true;

						jetapi.listQueueItems({ type: CONSTS.QUEUE_TYPE_RESTORE, snap_item_id: $scope.typeId }, function(response) {

							$scope.restores = [];

							if(!response.success) {
								SweetAlert.error($scope.LANG.t("Error!"), response.message);
								$scope.loadingRestores = false;
								return false;
							}

							for(var i = 0; i < response.data.queue.length; i++)
							{
								var data = response.data.queue[i];

								var text = '';
								if(data.status === 100) text = "Restore Completed";
								else if(data.status > 100) text = "Restore Failed";
								else text = "Restore in Progress";

								$scope.restores.push({
									_id: data._id,
									ready: data.status >= 100,
									status: data.status,
									created: data.created,
									text: $scope.LANG.t(text)
								});
							}

							$scope.loadingRestores = false;
							return true;
						});
					};

					$scope.inQueues = function(_id) {
						for(var i = 0; i < $scope.queues.length; i++) {
							if($scope.queues[i] == _id) return i;
						}

						return null;
					};

					$scope.checkStatus = function () {
						if(!PERMISSIONS.canViewQueue) return false;
						statusInterval = $interval(function() {

							var params = {};
							if($scope.type == 'destination') params.destination_id = $scope.typeId;
							else params.snap_item_id = $scope.typeId;

							jetapi.listQueueItems(params, function(response) {
								if (!response.success) {
									SweetAlert.error($scope.LANG.t("Error!"), response.message);
									return false;
								}

								var reload = false;
								var queue = response.data.queue;
								for(var i = 0; i < queue.length; i++) {
									var data  = queue[i];
									var index = $scope.inQueues(data._id);

									if(index !== null) {
										if(data.status >= 100) {
											$scope.queues.splice(index, 1);
											reload = true;
										}
									} else if(data.status < 100) {
										$scope.queues.push(data._id);
									}
								}

								if(reload) {
									$scope.getDownloads();
									$scope.getRestores();
								}

								return true;
							});

						}, 3000);
					};

					$scope.getDestination = function() {

						jetapi.getDestination({ _id: $scope.typeId }, function(response) {

							if(!response.success) {
								SweetAlert.error($scope.LANG.t("Error!"), response.message);
								return false;
							}

							$scope.typeDetails = response.data;
							return true;
						});
					};

					$scope.isDownloadsSupport = function() {
						if(!$scope.typeDetails.type) return true;
						var unsupportedDestinations = ['Backblaze', 'GoogleDrive', 'Dropbox'];
						return !(unsupportedDestinations.includes($scope.typeDetails.type));
					};

					$scope.getBackup = function() {

						jetapi.getBackup({ _id: $scope.typeId }, function(response) {

							if(!response.success) {
								SweetAlert.error($scope.LANG.t("Error!"), response.message);
								return false;
							}

							$scope.typeDetails = response.data;
							$scope.isDirectories = (CONSTS.BACKUP_TYPE_DIRECTORIES & $scope.typeDetails.type) != 0;
							return true;

						});
					};

					$scope.$on('$destroy', function() {
						if(angular.isDefined(statusInterval)) $interval.cancel(statusInterval);
					});

					$scope.init = function () {

						$scope.checkStatus();

						switch($scope.type)
						{
							case 'backup':
								$scope.getBackup();
								$scope.getRestores();
							break;
							case 'destination':
								$scope.getDestination();
							break;
						}

						$scope.getDownloads();
						$scope.fetch({path: '/'});
					};

					$timeout($scope.init());
				}]
		);

	}
);