'use strict';

define([
	'app',
	], function(app) {
	app.controller("queuePriorities",
		["$rootScope", "$scope", "$location", "$timeout", "$interval", "$q", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, $interval, $q, jetapi, meta, SweetAlert) {
				$rootScope.$emit('menuItem', 'Settings');

				$scope.priorities = [];
				$scope.loadingPriorities = false;
				$scope.loaders = {
					default: false,
					delete: false
				};

				meta = meta.new("queue_priorities");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("name");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["name", "priority"]);
				meta.setTotalItems($scope.priorities.length);

				$scope.onClickDelete = function(priority) {

					if($scope.saveing) return;
					$scope.saveing = true;

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("This queue priority will be permanently deleted!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm) {
							$scope.loaders.delete = true;
							jetapi.deleteQueuePriority({ _id: priority._id }, function(response) {
								$scope.loaders.delete = false;
								$scope.saveing = false;

								if(response.success) {
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
								}
								else {
									SweetAlert.error($scope.LANG.t("Failed!"), response.message);
								}
							});
						}
						else {
							$scope.saveing = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Queue Priority delete cancelled"));
						}

					});
				};

				$scope.setDefault = function (priority) {

					if($scope.saveing) return;
					$scope.saveing = true;

					$scope.loaders.default = true;
					jetapi.setDefaultQueuePriority({ _id: priority._id }, function(response) {
						$scope.saveing = false;
						$scope.loaders.default = false;

						if(response.success) {
							for(var i =0 ; i < $scope.priorities.length; i++) {
								if($scope.priorities[i].default) {
									$scope.priorities[i].default = 0;
									break;
								}
							}
							priority.default = 1;

							SweetAlert.success($scope.LANG.t("Success!"), response.message);
						}
						else {
							SweetAlert.error($scope.LANG.t("Failed!"), response.message);
						}
					});
				};



				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingPriorities = true;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						filter: meta.getFilter()
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.priorities = [];

					jetapi.listQueuePriorities(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.priorities);
						$scope.priorities = response.data.priorities;
						$scope.loadingPriorities = false;
					});
				};

				$timeout($scope.fetchData);
			}
		]
	);
});