'use strict';

define([
	'app',
	], function(app) {
	app.controller("schedulesList",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "jetapi", "meta", "SweetAlert", "type",
			function ($rootScope, $scope, $routeParams, $location, $timeout, jetapi, meta, SweetAlert, type) {
				//$rootScope.$emit('menuItem', 'Schedules');

				$scope.schedules = [];
				$scope.loadingSchedules = false;
				$scope.saveing = false;
				$scope.status = undefined;
				$scope.filter = 0;
				$scope.filterOptions = [];

				$scope.loaders = {
					delete: false
				};

				meta = meta.new("schedules");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("name");
				if(!meta.getSortDirection()) meta.setSortDirection("desc");
				meta.setSortFields(["name","type","owner","jobs_count"]);
				meta.setTotalItems($scope.schedules.length);

				$scope.onClickDelete = function(schedule) {

					if($scope.saveing) return;

					if(schedule.jobs_count > 0)
					{
						SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("This schedule has an assigned jobs, Please remove those jobs before deleting this schedule."));
						return false;
					}

					$scope.saveing = true;

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("This schedule will be permanently deleted!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.delete = true;

							jetapi.deleteSchedule({ _id: schedule._id }, function(response) {

								$scope.saveing = false;
								$scope.loaders.delete = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
								}
								else
								{
									SweetAlert.error($scope.LANG.t("Failed!"), response.message);
								}

							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Schedule delete cancelled"));
						}

					});
				};

				$scope.getScheduleIds = function(callback) {

					if(callback === undefined || typeof callback !== 'function') callback = function () {};

					if(!$routeParams.id) return callback([]);

					switch (type) {
						case 'backup':
							jetapi.getBackupJob({ _id: $routeParams.id }, function (response) {
								if(!response.success || !response.data.schedules.length) return callback([]);

								var schedules = [];
								for(var i = 0; i < response.data.schedules.length; i++) schedules.push(response.data.schedules[i]._id)
								return callback(schedules);
							});
						break;
					}

					return callback([]);
				};

				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingSchedules = true;
					$scope.status = undefined;
					$scope.schedules = [];

					$scope.getScheduleIds(function (schedules) {

						if(!schedules.length) {
							$scope.loadingSchedules = false;
							return;
						}

						var apiParams = {
							sort: {},
							skip: meta.getSkip(),
							limit: meta.getPageSize(),
							find: { _id: {$in: schedules} },
							filter: meta.getFilter()
						};

						if($scope.filter) apiParams.find.type = $scope.filter;
						apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

						jetapi.listSchedules(apiParams, function(response) {
							meta.setTotalItems(response.data.total);
							meta.calculate(response.data.schedules);
							$scope.schedules = response.data.schedules;
							$scope.loadingSchedules = false;
						});

					});

				};

				$timeout($scope.fetchData);
			}
		]
	);
});