'use strict';

define([
	'app',
], function (app) {
	app.controller("wizardSchedule",
		["$rootScope", "$scope", "$location", "$timeout", "$uibModal", "SweetAlert", "jetapi",
			function ($rootScope, $scope, $location, $timeout, $uibModal, SweetAlert, jetapi) {

				$scope.data = {
					time_parsed: '12:00 AM',
					time:0,
					schedules: [],
					retain: 14
				};

				$scope.service.isSaving = false;
				$scope.type = 'backup';

				$scope.init = function(){
					if($scope.jobWizard) {
						if($scope.wizardData.schedules) {
							for(var i=0; i<$scope.wizardData.schedules.length; i++)
								$scope.data.schedules.push($scope.wizardData.schedules[i]);
						}
						if($scope.wizardData.time_parsed) $scope.data.time_parsed = $scope.wizardData.time_parsed;
					}
					$scope.newScheduleFlag = false;

				}();

				$scope.$on($scope.wizardName, function () {

					if($scope.data.schedules.length < 1) {
						SweetAlert.swal({
							title: $scope.LANG.t("No Schedule is selected"),
							text: $scope.LANG.t("Are you sure you do not want to set up a schedule? This backup job is set to only run manually."),
							icon: "warning",
							buttons: [$scope.LANG.t("NO"), $scope.LANG.t("YES")]
						}, function (isConfirm) {
							if (isConfirm) {
								saveJobAndProceed();
							}
						});
					}
					else saveJobAndProceed();
				});

				var saveJobAndProceed = function() {
					$scope.service.isSaving = true;
					$scope.data.time = $scope.parseTime($scope.data.time_parsed);
					jetapi.manageBackupJob({ _id: $scope.wizardData.jobId, action: 'modify', schedules: $scope.data.schedules,	time: $scope.data.time, disabled:0 },
						function (response) {
							handleResponse(response);
						});
				};


				var handleResponse = function(response) {
					if(!response.success) {
						$scope.status = {
							message: response.message,
							type: "danger",
							closeable: false,
							ttl: 3000
						};
					}
					$scope.service.isSaving = false;
					$scope.process(response);
				};

				$scope.showSchedule = function () {

					$scope.newScheduleFlag = !$scope.newScheduleFlag;
					if($scope.status) $scope.status = null;

				};

				$scope.$on('scheduleResponse', function (event, response) {
					if(response.success) {
						jetapi.getSchedule({ _id: response.data._id }, function(details) {

							if(!details.success) {
								$scope.status = {
									message: details.message,
									type: "danger",
									closeable: false,
									ttl: 5000,
								};
								return;
							}
							if($scope.status) $scope.status = null;

							var currentSchedule = {
								_id: details.data._id,
								name: details.data.name,
								retain: $scope.data.retain
							};
							$scope.data.schedules.push(currentSchedule);
							$scope.newScheduleFlag = false;

						});
						return;
					}

					$scope.status = {
						message: response.message,
						type: "danger",
						closeable: false,
						ttl: 10000
					};
				});

				$scope.addSchedule = function(){

					if($scope.status) $scope.status = null;
					if($scope.data.retain <= 0) {
						$scope.status = {
							message: $scope.LANG.t("Invalid backups retain selected"),
							type: "danger",
							closeable: false,
							ttl: 10000
						};
						return;
					}
					$rootScope.$broadcast('createSchedule');
				};

				$scope.removeSchedule = function (schedule) {
					for (var i = 0; i < $scope.data.schedules.length; i++) {
						if (schedule._id === $scope.data.schedules[i]._id) {
							$scope.data.schedules.splice(i, 1);
							break;
						}
					}
				};

				$scope.parseTime = function (time) {
					var matches = /^([0-9]+):([0-9]+)\s+(AM+|PM+)$/g.exec(time);
					if (!matches) return 0;
					var output = parseInt(matches[1] + matches[2]);
					if (output >= 1200) output -= 1200;
					if (matches[3] === 'PM') output += 1200;
					return output;
				};
			}
		]
	);
});