'use strict';

define([
	'app',
	'libraries/permissions'
], function (app, PERMISSIONS) {
	app.directive('navigation', ["$rootScope", "$location", "lang", function($rootScope, $location, lang) {

		return {

			template:
				'<li aria-label="{{ option.label }}" ng-repeat="option in options" ng-if="option.permissions" ng-class="{ \'active\': active == option.class }" > ' +
					'<a href="{{url}}{{option.href}}" ng-click="changePath(option.href)" tooltip-placement="right" tooltip-enable="status == \'collapse\'" uib-tooltip="{{ option.label }}" > '+
						'<em aria-hidden="true" class="{{ option.icon }}"></em>' +
						'<span>{{ option.label }}</span>' +
					'</a>'+
				'</li>',
			scope: {
				"status": "=",
				"active": "="
			},
			link: function (scope) {

				scope.changePath = function(path) {
					$location.path(path)
				};

				scope.url = $rootScope.primaryURL;
				scope.options = [
					{label: lang.t("Dashboard"), 			href: "/", 				class: 'Dashboard', 	icon: 'fas fa-home', permissions: true },
					{label: lang.t("Accounts"), 			href: "/accounts", 		class: 'Accounts', 		icon: 'fas fa-users', permissions: PERMISSIONS.canViewAccounts },
					{label: lang.t("Destinations"), 		href: "/destinations", 	class: 'Destinations', 	icon: 'fas fa-folder', permissions: PERMISSIONS.canViewDestinations },
					{label: lang.t("Backup Jobs"),			href: "/backupJobs", 	class: 'BackupJobs', 	icon: 'fas fa-cubes', permissions: PERMISSIONS.canViewBackupJobs },
					{label: lang.t("Clone Jobs"), 			href: "/cloneJobs", 	class: 'CloneJobs', 	icon: 'fas fa-clone', permissions: PERMISSIONS.canViewCloneJobs },
					{label: lang.t("Restore & Download"),	href: "/backupManager", class: 'BackupManager', icon: 'fas fa-sync', permissions: PERMISSIONS.canViewAnyBackups },
					{label: lang.t("Alerts"), 				href: "/alerts", 		class: 'Alerts', 		icon: 'fas fa-exclamation-triangle', permissions: PERMISSIONS.canViewAlerts },
					{label: lang.t("Queues"), 				href: "/queues", 		class: 'Queues', 		icon: 'fas fa-clock', permissions: PERMISSIONS.canViewQueue },
					{label: lang.t("Logs"), 				href: "/logs", 			class: 'Logs', 			icon: 'fas fa-file-alt', permissions: PERMISSIONS.canViewLogs },
					{label: lang.t("Permissions"), 			href: "/permissions", 	class: 'Permissions', 	icon: 'fas fa-users-cog', permissions: PERMISSIONS.canViewAnyPermissions },
					{label: lang.t("Hooks"), 				href: "/hooks", 		class: 'Hooks', 		icon: 'fas fa-flag', permissions: PERMISSIONS.canViewHooks },
					{label: lang.t("Security"), 			href: "/security", 		class: 'Security', 		icon: 'fas fa-fingerprint', permissions: PERMISSIONS.isRoot },
					{label: lang.t("GDPR"), 				href: "/settings/gdpr", class: 'GDPR', 			icon: 'fas fa-lock', permissions: PERMISSIONS.isRoot },
					{label: lang.t("Settings"), 			href: "/settings", 		class: 'Settings', 		icon: 'fas fa-cog', permissions: PERMISSIONS.isRoot }
					//{label: lang.t("Schedules"), href: "#!/schedules", class: 'Schedules', icon: 'fa fa-calendar', permissions: PERMISSIONS.canViewSchedules },
					//{label: lang.t("Account Filters"), href: "#!/accountFilters", class: 'AccountFilters', icon: 'fa fa-home', permissions: PERMISSIONS.canViewAccountFilters },
				];
			}
		};
	}]);
});

